/*
HARDWARE
868 MHz emonTx V2

*/

#include <Arduino.h>
#include "emonLibCM.h"

#define RF69_COMPAT 0                            //  Set to 1 if using RFM69CW, or 0 if using RFM12B
#include <JeeLib.h>                              //  https://github.com/jcw/jeelib - Tested with JeeLib 4 Nov 2016
// ISR(WDT_vect) { Sleepy::watchdogEvent(); } 

#define RF_freq RF12_868MHZ                      //  Frequency of RF12B module can be RF12_433MHZ, RF12_868MHZ or RF12_915MHZ. You must use the one matching the module you have.
const int nodeID = 9;                            //  node ID for this emonTx. This sketch does NOT interrogate the DIP switch.

const int networkGroup = 210;                    //  wireless network group
                                                 //  - needs to be same as emonBase / emonPi and emonGLCD. OEM default is 210



typedef struct {unsigned long msgnum; int power, VA, pf, Vrms; long WH; unsigned long pulsecount; int temp; } PayloadTX;        // neat way of packaging data for RF comms

PayloadTX emontx;                                // create an instance


DeviceAddress allAddresses[3];                             // Array to receive temperature sensor addresses
int allTemps[3];                                           // Array to receive temperature measurements

/*
DeviceAddress allAddresses[8] = {       
    {0x28, 0x81, 0x43, 0x31, 0x7, 0x0, 0x0, 0xD9}, 
    {0x28, 0x8D, 0xA5, 0xC7, 0x5, 0x0, 0x0, 0xD5},         // Use the actual addresses, as many as required
    {0x28, 0xC9, 0x58, 0x32, 0x7, 0x0, 0x0, 0x89}          //  up to a maximum of 6
    };

// Use the actual addresses, as many as required */

 
#define LEDpin 9

 
void setup() 
{  
 
  Serial.begin(9600);
  Serial.println("Set baud=115200");
  Serial.end();
  Serial.begin(115200);
  
  Serial.println("EmonTx V2 EmonLibCM Continuous Monitoring Tester - 868"); 

  EmonLibCM_SetADC_VChannel(2, 266.98);                    // ADC Input channel, calibration for Ideal UK Adapter = 268.97 
  EmonLibCM_SetADC_IChannel(3, 86.163, 0.5);               // ADC Input channel, current calibration, phase calibration
  EmonLibCM_SetADC_IChannel(0, 86.163, 0.5);               // ADC Input channel, current calibration, phase calibration
  EmonLibCM_SetADC_IChannel(1, 86.163, 0.5);               // ADC Input channel, current calibration, phase calibration
 

  EmonLibCM_setADC(10, 104);                               // ADC Bits (10 for emonTx & Arduino except Due=12 bits, ADC Duration 104 us for 16 MHz operation)
  EmonLibCM_cycles_per_second(50);                         // mains frequency 50Hz, 60Hz
  EmonLibCM_datalog_period(10);                            // period of readings in seconds - normal value for emoncms.org
  
  EmonLibCM_min_startup_cycles(10);                        // number of cycles to let ADC run before starting first actual measurement


  EmonLibCM_setPulseEnable(true);                          // Enable pulse counting

  EmonLibCM_setPulsePin(3, 1);
  EmonLibCM_setPulseMinPeriod(100);

                                                 
  EmonLibCM_ADCCal(3.3);                                   // ADC Reference voltage, (3.3 V for emonTx,  5.0 V for Arduino)
  
  EmonLibCM_setTemperatureDataPin(4);                      // OneWire data pin (emonTx V2)
  EmonLibCM_setTemperaturePowerPin(-1);                    // Temperature sensor Power Pin - 19 for emonTx V3.4  (-1 = Not used. No sensors, or sensor are permanently powered.)
  EmonLibCM_setTemperatureResolution(11);                  // Resolution in bits, allowed values 9 - 12. 11-bit resolution, reads to 0.125 degC
  EmonLibCM_setTemperatureAddresses(allAddresses);         // Name of array of temperature sensors
  EmonLibCM_setTemperatureArray(allTemps);                 // Name of array to receive temperature measurements
  EmonLibCM_setTemperatureMaxCount(8);                     // Max number of sensors, limited by wiring and array size.
  
  EmonLibCM_TemperatureEnable(true); 
  printTemperatureSensorAddresses();                       // Show which sensors are connected

  rf12_initialize(nodeID, RF_freq, networkGroup);          // initialize radio module

  pinMode(LEDpin, OUTPUT);                                 // Setup indicator LED

  EmonLibCM_Init();                                        // Start continuous monitoring.
  
}

void loop()             
{

  static unsigned long msgnum = 0;
  
  if (EmonLibCM_Ready())   
  {
    int ct = 1;


    Serial.println(EmonLibCM_acPresent()?"AC present ":"AC missing ");
    delay(5);

	  double Vrms = EmonLibCM_getVrms();

    emontx.msgnum = msgnum++;
    emontx.power  = EmonLibCM_getRealPower(EmonLibCM_getLogicalChannel(3));   // Copy the desired variables ready for transmission
	  emontx.VA     = EmonLibCM_getApparentPower(0);
	  emontx.pf     = EmonLibCM_getPF(0) * 10000;
	  emontx.Vrms   = Vrms * 100;
    emontx.WH     = EmonLibCM_getWattHour(0);
    emontx.pulsecount = EmonLibCM_getPulseCount();
    emontx.temp   =  allTemps[0];


    digitalWrite(LEDpin, true); delay(20); digitalWrite(LEDpin, false);
    
    send_rf_data();

    delay(50);
    
    Serial.print(" V=");Serial.println(Vrms);
    for (ct=0; ct<3; ct++)
    {
      Serial.print(" I=");Serial.print(EmonLibCM_getIrms(ct),3);
      Serial.print(" W=");Serial.print(EmonLibCM_getRealPower(ct));
      Serial.print(" VA=");Serial.print(EmonLibCM_getApparentPower(ct));
      Serial.print(" Wh=");Serial.print(EmonLibCM_getWattHour(ct));
      Serial.print(" pf=");Serial.println(EmonLibCM_getPF(ct),4);   
      delay(10);
    } 
        
    for (byte t=0; t<8; t++) 
    {
      Serial.print(" T=");Serial.println(EmonLibCM_getTemperature(t),4);    
    }
    Serial.print(" pulses=");Serial.println(EmonLibCM_getPulseCount());

    delay(10);
        
  }
  else
    rf12_recvDone();
}

void send_rf_data()
{
  rf12_sleep(RF12_WAKEUP);
  rf12_sendNow(0, &emontx, sizeof emontx);                           //send data via RFM12B using new rf12_sendNow wrapper
  rf12_sendWait(2);
}
